package cz.muni.fi.pb162.school;

import java.util.Collection;
import java.util.Set;

/**
 * Toto rozhrani reprezentuje katalog predmetu a studentu. Umoznuje 
 * vytvaret nove kurzy a studenty a umoznuje jednotlivym studentum
 * si zapisovat kurzy.
 * 
 * @author Petr Adamek &lt;xadamek2@fi.muni.cz&gt;
 */
public interface Catalog {

    /**
     * Metoda vytvori noveho studenta s danym jmenem. Student bude 
     * mit pridelen novy identifikator, ktery jeste zadny student
     * v kolekci nema.
     *
     * @param name jmeno noveho studenta
     * @return instanci tridy reprezentujici noveho studenta
     * @throws java.lang.NullPointerException pokud ma paramentr name 
     * hodnotu null
     */
    Student newStudent(String name);
    
    /**
     * Vrati studenta s danym id.
     *
     * @param id Identifikator studenta, ktereho ma metoda vratit.
     * @return instanci tridy reprezentujici studenta s danym identifikatorem 
     * nebo null, pokud takovy student neexistuje.
     */
    Student getStudent(int id);
    
    
    /**
     * Vrati kolekci vsech studentu v katalogu. Kolekci neni mozne 
     * modifikovat. Vsimnete si, ze tato metoda nikdy nevraci hodnotu null! 
     *
     * @return mnozinu vsech studentu v katalogu
     */
    Collection<Student> getStudents();

    /**
     * Vytvori novy kurz. Jeho identifikator nesmi obsahovat mezeru.
     * 
     * 
     * @param id kod noveho kurzu
     * @param name jmeno noveho kurzu
     * @return nove vytvoreny kurz
     * @throws cz.muni.fi.pb162.school.CatalogException Pokud kurz s 
     * danym kodem jiz existuje
     * @throws java.lang.NullPointerException Pokud ma parametr id nebo name
     * hodnotu null
     * @throws java.lang.IllegalArgumentException Pokud parametr id obsahuje
     * mezeru
     */
    public Course newCourse(String id, String name) throws 
            CatalogException;
    
    /**
     * Vrati kolekci vsech predmetu v katalogu. Kolekci neni mozne 
     * modifikovat. Vsimnete si, ze tato metoda nikdy nevraci hodnotu null! 
     *
     * @return kolekci vsech predmetu v katalogu
     */
    public Collection<Course> getCourses();

    /**
     * Smaze vsechny studenty i predmety v katalogu.
     */
    void clear();

    /**
     * Vrati seznam kurzu, ktere ma dany student zapsane
     * 
     * 
     * @param student student, jehoz kurzy nas zajimaji
     * @return mnozinu zapsanych kurzu
     * @throws cz.muni.fi.pb162.school.CatalogException pokud student 
     * neexistuje
     */
    public Set<Course> getCourses(Student student) throws 
            CatalogException;

    /**
     * Vrati seznam studentu, kteri maji zapsany dany kurz.
     * 
     * @param course kurz, jehoz studenti nas zajimaji
     * @return seznam studentu, kteri maji kurz zapsany
     * @throws cz.muni.fi.pb162.school.CatalogException pokud dany kurz neexistuje
     */
    public Set<Student> getStudents(Course course) throws 
            CatalogException;


    /**
     * Zapise studenta do zadanych kurzu
     * 
     * @param student student, ktery se bude zapisovar
     * @param courses kurzy, do nichz ma byt student zapsan
     * @throws cz.muni.fi.pb162.school.CatalogException 
     * pokud je student jiz do nektereho z uvedenych kurzu zapsan, 
     * nebo pokud dany kurz nebo student neexistuje.
     * @throws java.lang.NullPointerException 
     * pokud ma nektery z parametru hodnotu null
     */
    public void register(Student student, Course... courses) throws 
            CatalogException;
    
}
