package cz.muni.fi.pb162.school;

import java.io.InputStream;
import java.io.OutputStream;

/**
 * Tato trida umoznuje ulozit katalog studentu a predmetu
 * do binarniho proudu a pak jej z tohoto proudu opet nacist.
 * 
 * Katalog je v proudu (souboru) organizovan tak, ze nejdrive obsahuje seznam 
 * studentu. Na kazdem radku je jeden student, pricemz radek zacina 
 * identifikatorem studenta, pak nasleduje mezera a zbytek radku obsahuje 
 * jmeno studenta. Konec seznamu studentu je indikovan jednim 
 * prazdnym radkem.
 *
 * Pak nasleduje seznam predmetu. Kazdy kurs je ulozen na dvou radcich, 
 * pricemz prvni obsahuje kod predmetu, pak mezeru a zbytek radku tvori 
 * nazev. Na dalsim radku jsou mezerou oddelene id studentu, kteri maji
 * dany predmet zaregistrovany. Id studentu jsou usporadana od nejmensiho 
 * po nejvetsi.
 *
 * Data jsou v proudu ulozena v kodovani UTF-8.
 *
 * @author Petr Adamek &lt;xadamek2@fi.muni.cz&gt;
 *
 */
public interface CatalogStorage  {
    
    /**
     * Tato metoda ze zadaneho proudu nacte obsah katalogu. 
     * 
     * @param is Vstupni proud, ze ktereho se budou cist data.
     * @throws CatalogException pokud dojde k chybe pri cteni dat z 
     * proudu nebo ma vstupni soubor nespravny format.
     */
    public Catalog load(InputStream is) throws CatalogException;

    /**
     * Tato metoda obsah katalogu ulozi do daneho vystupniho proudu. 
     * 
     * @param catalog Katalog, jehoz obsah bude zapsan do vystupniho proudu.
     * @param os Vystupni proud, do nejz se ma katalog zapsat.
     * @throws CatalogException pokud dojde k chybe pri zapisu dat do proudu
     */
    public void save(Catalog catalog, OutputStream os) throws CatalogException;
    
}
