package cz.muni.fi.pb162.school;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Map;
import java.util.Set;
import java.util.TreeMap;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 *
 * @author Petr Adamek
 */
public class CatalogStorageTest extends TestCase {
    
    public CatalogStorageTest(String testName) {
        super(testName);
    }
    
    public static junit.framework.Test suite() {
        TestSuite suite = new TestSuite(CatalogStorageTest.class);
        return suite;
    }
    
    protected void assertException(Throwable t) {
        assertTrue("Vyjimka " + t.getClass().getName() +
                " neobsahuje podrobny popis!",
                t.getMessage() != null &&
                t.getMessage().trim().length() > 0);
    }
        
    private static final String[] TEST_DATA_COURSES = {
        "PB162 Programovani v jazyce Java",
                "1 2",
                "PA165 Vyvoj programovych systemu v jazyce Java",
                "1",
                "XXX Franti\u0161ek \u017dlu\u0165ou\u010dk\u00fd-Kon\u00ed\u010dek",
                ""
    };
    
    protected static final String[] TEST_DATA_STUDENTS = {
        "1 Petr Ad\u00e1mek",
                "2 Mario Martinez Chuares Ferdinando",
                "3 Don Quijote De La Mancha",
                "12384 Johanka z Arku",
                "4 Franti\u0161ek \u017dlu\u0165ou\u010dk\u00fd-Kon\u00ed\u010dek"
    };

    protected static final String[] EMPTY_LINE = { "" };

    public void testRules() {
	BasicRulesTester.testMethodsAndAttributes(CatalogStorageImpl.class);
    }


    public final void testLoadStudents() throws IOException, CatalogException {

        Catalog catalog = new CatalogStorageImpl().load(
                createTestInputStream(TEST_DATA_STUDENTS, EMPTY_LINE));

        assertEquals("Katalog po nacteni dat obsahuje nespravny pocet studentu",
            TEST_DATA_STUDENTS.length, catalog.getStudents().size());

        Map<Integer,String> expectedStudents = new TreeMap<Integer,String>();
        Map<Integer,String> loadedStudents = new TreeMap<Integer,String>();
        
        for (String line : TEST_DATA_STUDENTS) {
            String[] columns = line.split(" ",2);
            int id = Integer.parseInt(columns[0]);
            String name = columns[1];
            expectedStudents.put(id,name);
            
            Student student = catalog.getStudent(id);
            assertNotNull("Metoda getStudent nenalezne studenta s id " + id +
                    ", ktery je obsazen v testovacim souboru",student);
            assertEquals("Student s id " + id + "vraceny metodou getStudent " +
                    "obsahuje jine jmeno, nez jake ma v testovacim souboru",
                    name,student.getName());
            assertEquals("Student s id " + id + "vraceny metodou getStudent " +
                    "obsahuje jine id, nez jake ma v testovacim souboru",
                    id,student.getId());
            
            try {
                assertNotNull("Metoda getCourses pro studenta s id " + id +
                        "(ktery je obsazen v testovacim souboru) vraci null" +
                        " misto prazdne mnoziny",catalog.getCourses(student));
            } catch(Exception ex) {
                fail("Metoda getCourses pro studenta s id " + id +
                        "(ktery je obsazen v testovacim souboru) vyvola vyjimku " +
                        ex.getClass().getName());
            }
        }

        for (Student student : catalog.getStudents()) {
            loadedStudents.put(student.getId(),student.getName());
        }
        
        assertEquals("Mno\u017eina na\u010dten\u00fdch student\u016f se li\u0161\u00ed od o\u010dek\u00e1van\u00fdch dat",
                expectedStudents, loadedStudents);
        
        int nextStudentId = catalog.newStudent("test x").getId();
        assertFalse("Metoda newStudent pridelila novemu studentovi id (" +
                nextStudentId +"), ktere jiz ma nektery ze studentu " +
                "v testovacim souboru pridelene",
                expectedStudents.keySet().contains(nextStudentId));
    }
    
    public void testLoadAll() throws IOException, CatalogException {

        Catalog catalog = new CatalogStorageImpl().load(createTestInputStream(
                TEST_DATA_STUDENTS,EMPTY_LINE,TEST_DATA_COURSES));

        assertEquals("Katalog po nacteni dat obsahuje nespravny pocet kurzu",
                TEST_DATA_COURSES.length / 2, catalog.getCourses().size());

        Map<String,String> expectedCourses = new TreeMap<String,String>();
        Map<String,String> loadedCourses = new TreeMap<String,String>();

        // klicem je id kurzu, hodnotou tento kurz
        Map<String,Course> coursesMap = new HashMap<String,Course>();

        // klicem je id kurzu, hodnotou seznam studentu
        Map<String,Set<Student>> expectedRegisteredStudents =
            new HashMap<String,Set<Student>>();

        // klicem je id studenta, hodnotou seznam kurzu
        Map<Integer,Set<Course>> expectedRegisteredCourses =
            new HashMap<Integer,Set<Course>>();

        for (Course course : catalog.getCourses()) {
            loadedCourses.put(course.getId(),course.getName());
            coursesMap.put(course.getId(),course);
        }

        for (Student student : catalog.getStudents()) {
            expectedRegisteredCourses.put(student.getId(),new HashSet<Course>());
        }

        for (int i = 0; i<TEST_DATA_COURSES.length; ) {
            String line = TEST_DATA_COURSES[i++];
            String[] columns = line.split(" ",2);
            String id = columns[0];
            String name = columns[1];
            expectedCourses.put(id,name);

            Set<Student> registeredStudents = new HashSet<Student>();
            columns = TEST_DATA_COURSES[i++].split(" ");
            for(String c : columns) {
                if (c.length() > 0) {
                    Integer studentId = Integer.valueOf(c);
                    registeredStudents.add(catalog.getStudent(studentId));
                    // overime, ze nejde o neznameho studenta
                    if (expectedRegisteredCourses.containsKey(studentId)) {
                        expectedRegisteredCourses.get(studentId).add(coursesMap.get(id));
                    }
                }
            }
            expectedRegisteredStudents.put(id, registeredStudents);
        }

        assertEquals("Mnozina nactenych kurzu se lisi od ocekavanych dat",
                expectedCourses, loadedCourses);

        for (Course course : catalog.getCourses()) {
            assertEquals("Mnozina studentu registrovanych v predmetu " +
                    course.getId() + " neodpovida vstupnim datum",
                    expectedRegisteredStudents.get(course.getId()),
                    catalog.getStudents(course));
        }

        for (Student student : catalog.getStudents()) {
            assertEquals("Mnozina registrovanych predmetu studenta " +
                    student + " neodpovida vstupnim datum",
                    expectedRegisteredCourses.get(student.getId()),
                    catalog.getCourses(student));
        }

    }

    public void testLoadCoursesWithUnknownStudent() throws 
            IOException, CatalogException {

        try {
            new CatalogStorageImpl().load(createTestInputStream(
                TEST_DATA_STUDENTS, EMPTY_LINE, new String[] {"XXX test","12"}));
            fail("Je mozne nahrat ze vstupniho souboru kurz, ktery ma " +
                    "zapsany neexistujici student");
        } catch (CatalogException ex) {
            // All is ok
            assertException(ex);
        } catch (Exception ex) {
            fail("Pokus o nahrati kurzu s neexistujicim studentem vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }

    }

    public void testLoadCoursesAndRegister() throws IOException, CatalogException {
        Catalog catalog = new CatalogStorageImpl().load(
                createTestInputStream(TEST_DATA_STUDENTS,EMPTY_LINE,TEST_DATA_COURSES));

        catalog.register(catalog.getStudent(3),catalog.getCourses().iterator().next());
    }

    public void testSave() throws IOException, CatalogException {

        Catalog catalog = new CatalogStorageImpl().load(
                createTestInputStream(TEST_DATA_STUDENTS,EMPTY_LINE,TEST_DATA_COURSES));

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
        new CatalogStorageImpl().save(catalog,outputStream);

        // Testovaci data ulozit do pomocnych kolekci
        Set<String> testDataStudentsLines = new HashSet<String>();
        for (String line : TEST_DATA_STUDENTS) {
                testDataStudentsLines.add(line);
        }
        Set<String> testDataCoursesLines = new HashSet<String>();
        for (int i = 0; i < TEST_DATA_COURSES.length;) {
            testDataCoursesLines.add(TEST_DATA_COURSES[i++] + 
                    ", " + TEST_DATA_COURSES[i++]);
        }

        // otestujeme studenty
        BufferedReader reader = new BufferedReader(new InputStreamReader(
        new ByteArrayInputStream(outputStream.toByteArray()), ENCODING));

        while (true) {
            String line = reader.readLine();
            assertNotNull("Vystupni soubor neocekavane skoncil (nacitam prave " +
                    "studenty a ocekavam na konci prazdny radek)",line);
            if (line.length() == 0) { break; }
            assertTrue("Vystupni soubor obsahuje radek, ktery nebyl ve vstupnich" +
                    " testovacich datech: " + line,testDataStudentsLines.contains(line));
            testDataStudentsLines.remove(line);
        }
        assertTrue("Ve vystupnim souboru chybi nektere radky ze vstupnich " +
                "testovacich dat: " + testDataStudentsLines,
                testDataStudentsLines.isEmpty());
        
        String line;
        while (null != (line = reader.readLine())) {
            String line2 = reader.readLine();
            assertNotNull("Vystupni soubor neocekavane skoncil (u posledniho " +
                    "predmetu chybi seznam studentu)",line2);
            line = line + ", " + line2;
            assertTrue("Vystupni soubor obsahuje dvojici radku, ktera nebyla ve vstupnich" +
                    " testovacich datech: " + line + "(mozna jenom nejsou serazene id " +
                    "studentu)",testDataCoursesLines.contains(line));
            testDataCoursesLines.remove(line);
        }
        assertTrue("Ve vystupnim souboru chybi nektere radky ze vstupnich " +
                "testovacich dat: " + testDataCoursesLines,
                testDataCoursesLines.isEmpty());
        
    }
    
    public final void testBadStudentDataNoName() throws IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(
                    createTestInputStream(new String[] { "128", "" }));
                    fail("Metoda load nevyvola vyjimku CatalogException, " +
                            "ackoliv je ve vstupnim souboru radek bez jmena studenta");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load na radku bez jmena studenta vyvola misto vyjimky " +
                    "CatalogException vyjimku " + ex.getClass().getName());
        }
    }
    
    public final void testBadStudentDataInvalidNumber() throws IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(
                    createTestInputStream(new String[] { "1a jmeno", "" }));
                    fail("Metoda load nevyvola vyjimku CatalogException, " +
                            "ackoliv je ve vstupnim souboru radek neciselnym id");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load na radku s neciselnym id vyvola misto vyjimky " +
                    "CatalogException vyjimku " + ex.getClass().getName());
        }
    }

    public final void testBadStudentDataDuplicateId() throws 
            IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(createTestInputStream(
                    new String[] { "1 jmeno", "2 xxx", "1 hehehe", "" }));
            fail("Metoda load nevyvola vyjimku CatalogException, " +
                "ackoliv je ve vstupnim souboru vice studentu se stejnym id");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load u souboru s vice studenty se stejnym id vyvola " +
                    "misto vyjimky CatalogException vyjimku " + 
                    ex.getClass().getName());
        }
    }
    
    public void testBadCourseDataEmptyLine() throws IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(createTestInputStream(
                    new String[] { "1 Petr Adamek", "", "" }));
                    fail("Metoda load nevyvola vyjimku CatalogException, " +
                            "ackoliv je ve vstupnim souboru v prazdny radek misto " +
                            "definice predmetu");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load na prazdnem radku s definici predmetu vyvola " +
                    "misto vyjimky CatalogException vyjimku " +
                    ex.getClass().getName());
        }
        
        try {
            new CatalogStorageImpl().load(createTestInputStream(new String[] {
                "1 Petr Adamek", "",
                 "PB138 Modern\u00ed zna\u010dkovac\u00ed jazyky a jejich aplikace", "",
                 "", "1"}));
            fail("Metoda load nevyvola vyjimku CatalogException, " +
                    "ackoliv je ve vstupnim souboru v prazdny radek misto " +
                    "definice predmetu");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load na prazdnem radku s definici predmetu vyvola " +
                    "misto vyjimky CatalogException vyjimku " +
                    ex.getClass().getName());
        }
    }
    
    public void testBadCourseDataNoName() throws IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(createTestInputStream(
                    new String[] { "1 Petr Adamek", "", "128", "1" }));
            fail("Metoda load nevyvola vyjimku CatalogException, " +
                    "ackoliv je ve vstupnim souboru radek bez jmena predmetu");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load na radku bez jmena predmetu vyvola misto vyjimky " +
                    "CatalogException vyjimku " + ex.getClass().getName());
        }
    }

    public void testBadCourseDataDuplicateId() throws IOException, CatalogException {
        
        try {
            new CatalogStorageImpl().load(createTestInputStream(new String[] 
                { "1 Petr Adamek", "", "PB138 xxx", "1", "PB138 xxc", "1"}));
            fail("Metoda load nevyvola vyjimku CatalogException, " +
                    "ackoliv je ve vstupnim souboru vice predmetu se stejnym kodem");
        } catch(CatalogException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
            assertException(ex);
        } catch(Exception ex) {
            // jakakoliv jina vyjimka je chybou
            fail("Metoda load u souboru s duplicitnimi kody predmetu vyvola " +
                    "misto vyjimky CatalogException vyjimku " + 
                    ex.getClass().getName());
        }
    }

    public void testBadCoursesMissingLastNumber() {
        try {
            new CatalogStorageImpl().load(createTestInputStream(
                    new String[] {"1 Petr Adamek","","PB162 Java"}));
            fail("Soubor s chybejicim seznamem studentu u posledniho predmetu " +
                    "nevyvola zadnou vyjimku");
        } catch (CatalogException ex) {
            // All is ok
            assertException(ex);
        } catch (Exception ex) {
            fail("Pokus o nacteni souboru s chybejicim seznamem studentu u " +
                    "posledniho predmetu vyvola misto CatalogException " +
                    "vyjimku " + ex.getClass().getName());
        }
    }

    public void testBadCoursesInvalidNumber() {
        try {
            new CatalogStorageImpl().load(createTestInputStream(
                    new String[] {"1 Petr Adamek","","PB162 Java", "1as"}));
            fail("Soubor se spatnym formatem id studenta u predmetu " +
                    "nevyvola zadnou vyjimku");
        } catch (CatalogException ex) {
            // All is ok
            assertException(ex);
        } catch (Exception ex) {
            fail("Pokus o nacteni souboru se spatnym formatem id studenta u " +
                    "predmetu vyvola misto CatalogException vyjimku " + 
                    ex.getClass().getName());
        }
    }
    
    public final void testLoadIOError() {
        try {
            new CatalogStorageImpl().load(new BadInputStream());
            fail("Vyjimka IOException pri cteni ze vstupniho proudu" +
                    " je zrejme ignorovana!!! " +
                    "(nezpusobi vyvolani pozadovane vyjimky CatalogException)");
        } catch (CatalogException ex) {
            assertSame("Vyjimka IOException pri cteni ze vstupniho proudu " +
                    "sice zpusobi vyvolani vyjimky CatalogException, ale tato " +
                    "vyjimka nema spravne nastavenou svoji pricinu (viz retezeni vyjimek)",
                    IOEXCEPTION,ex.getCause());
        } catch (Exception ex) {
            fail("Vyjimka IOException pri cteni ze vstupniho proudu" +
                    " zpusobi vyvolani vyjimky " + ex.getClass() + " misto pozadovane " +
                    "CatalogException");
        }
    }
    
    public final void testSaveIOError() {
        Catalog catalog = new CatalogImpl();
        catalog.newStudent("Test student");
        CatalogStorage storage = new CatalogStorageImpl();
        try {
            storage.save(catalog,new BadOutputStream());
            fail("Vyjimka IOException pri zapisu do vystupniho proudu" +
                    " je zrejme ignorovana!!! " +
                    "(nezpusobi vyvolani pozadovane vyjimky CatalogException). " +
                    "Nepouzivate nahodou tridu PrintWriter, ktera IOException " +
                    "negeneruje?");
        } catch (CatalogException ex) {
            assertSame("Vyjimka IOException pri zapisu do vystupniho proudu " +
                    "sice zpusobi vyvolani vyjimky CatalogException, ale tato " +
                    "vyjimka nema spravne nastavenou svoji pricinu (viz retezeni vyjimek)",
                    IOEXCEPTION,ex.getCause());
        } catch (Exception ex) {
            fail("Vyjimka IOException pri zapisu do vystupniho proudu" +
                    " zpusobi vyvolani vyjimky " + ex.getClass() + " misto pozadovane " +
                    "CatalogException");
        }
    }
    
    public static final String ENCODING = "UTF-8";
    
    protected static byte[] stringLinesToByteArray(String[]... lines) throws IOException {
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        Writer writer = new OutputStreamWriter(out,ENCODING);
        for (String[] someLines : lines) {
            for (String line : someLines) {
                writer.write(line);
                writer.write("\n");
            }
        }
        writer.flush();
        return out.toByteArray();
    }
    
    protected static InputStream createTestInputStream(String[]... lines) throws 
            IOException {
        return new ByteArrayInputStream(stringLinesToByteArray(lines));
    }
    
    protected static final IOException IOEXCEPTION =
            new IOException("It is not possible to read from this stream");
    
    protected class BadInputStream extends InputStream {
        
        public int read() throws IOException {
            throw IOEXCEPTION;
        }
    }
    
    protected class BadOutputStream extends OutputStream {
        
        public void write(int value) throws IOException {
            throw IOEXCEPTION;
        }
    }
    
}
