package cz.muni.fi.pb162.school;

import java.util.HashSet;
import java.util.Set;
import junit.framework.Test;
import junit.framework.TestCase;
import junit.framework.TestSuite;

/**
 * Testovací třída pro katalog studentů
 *
 * @author Petr Adamek &lt;xadamek2@fi.muni.cz&gt;
 */
public class CatalogTest extends TestCase {
    
    public CatalogTest(String testName) {
        super(testName);
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(CatalogTest.class);
        return suite;
    }
    
    protected void assertException(String when, Throwable t) {
        assertTrue("Vyjimka " + t.getClass().getName() + " vyvolana pri " +
                "pokusu o " + when + " neobsahuje podrobny popis!",
                t.getMessage() != null &&
                t.getMessage().trim().length() > 0);
    }

    public void testRules() {
	BasicRulesTester.testMethodsAndAttributes(StudentImpl.class);
	BasicRulesTester.testMethodsAndAttributes(CourseImpl.class);
	BasicRulesTester.testMethodsAndAttributes(CatalogImpl.class);
    }
    
    public final void testStudentEquals() {
        Student student = new CatalogImpl().newStudent("Test student");
        assertNotNull("Metoda newStudent vraci null",student);

        assertTrue("Metoda equals vraci false, i kdyz je jejim parametrem " +
        "instance studenta se stejnym id (chyba v metode StudentImpl.equals)",
        student.equals(new TestStudent(student.getId())));
        assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
        "instance studenta s odlisnym id (chyba v metode StudentImpl.equals)",
        student.equals(new TestStudent(student.getId() + 1)));

        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
                    "hodnota null (chyba v metode StudentImpl.equals)",
                    student.equals(null));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda StudentImpl.equals() s parametrem null," 
                    + " metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }

        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
                    "objekt jineho typu (chyba v metode StudentImpl.equals)",
                    student.equals(new Object()));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda StudentImpl.equals() s parametrem " +
                    "jineho typu, metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }
        
        assertFalse("Metoda StudentImpl.hashCode() neni prekryta, takze " +
                "porusuje obecny kontrakt metody Object.hashCode()",
                student.hashCode() == System.identityHashCode(student));
        
        Catalog catalog = new CatalogImpl();
        Student student2 = catalog.newStudent("Test student");
        assertEquals("Metoda CatalogImpl.newStudent se chova " +
        "nedeterministicky; prvni vytvoreny student ma pokazde jine id",
        student.getId(),student2.getId());
        
        assertEquals("Metoda StudentImpl.hashCode() vraci pro dva stejne " +
                "studenty ruznou hodnotu, takze porusuje obecny kontrakt " +
                "metody Object.hashCode() ",
                student.hashCode(),student2.hashCode());

        assertFalse("Metoda StudentImpl.hashCode() vraci pro dva ruzne " +
                "studenty stejny kod; to sice neporusuje obecny kontrakt " +
                "metody Object.hashCode(), ale v tomto pripade lze snadno " +
                "vytvorit bezkolizni implementaci teto metody",
                student.hashCode() == 
                catalog.newStudent("Test student 2").hashCode());        
    }

    private static void assertSameSign(String msg, int n1, int n2) {
        assertTrue(msg,
                // musi byt osetrena i nula, proto zdanlive 
                // zbytecna druha cast nasledujiciho vyrazu
                ((n1 < 0) == (n2 < 0)) && ((n1 > 0) == (n2 > 0)));
    }
    
    public final void testStudentCompareTo() {

        Catalog catalog = new CatalogImpl();
        Student s1 = catalog.newStudent("Test student 1");
        Student s2 = catalog.newStudent("Test student 1");
        
        String msg = "Metoda StudentImpl.compareTo() nezajisti usporadani " +
                "podle id";
        assertSameSign(msg, s1.getId() - s2.getId(), s1.compareTo(s2));
        assertSameSign(msg, s2.getId() - s1.getId(), s2.compareTo(s1));
        
        assertEquals("Metoda StudentImp.compareTo() se nechova spravne, kdyz " +
                "porovnava dva stejne objekty",0,s1.compareTo(s1));
        assertEquals("Metoda StudentImp.compareTo() se nechova spravne, kdyz " +
                "porovnava dva stejne objekty",0,
                s1.compareTo(new TestStudent(s1.getId())));
    }

    public final void testNewStudent() {
        Catalog catalog = new CatalogImpl();
        Student student = catalog.newStudent("Test student");
        assertNotNull("Metoda newStudent vraci null",student);

        assertEquals("Nove vytvoreny student ma jine jmeno, nez bylo zadano " +
        "jako parametr metody CatalogImpl.newStudent()","Test student",
        student.getName());
        
        Student student2 = catalog.getStudent(student.getId());
        assertNotNull("Metoda getStudent s identifikatorem nove vytvoreneho " +
        "studenta vraci null",student2);
        
        assertTrue("Mnozina studentu vracena metodou getStudents() neobsahuje" +
        " nove vytvoreneho studenta",catalog.getStudents().contains(student));

        try {
             catalog.newStudent(null);
             fail("Zavolani metody newStudent s parametrem name s hodnotou " +
                     "null nezpusobi vyvolani pozadovane vyjimky " +
                     "NullPointerException");
        } catch(NullPointerException ex) {
            // Vse je v poradku
            assertException("zavolani metody newStudent s parametrem name s " +
                    "hodnotou null",ex);
        }
    }

    public void testCourseEquals() throws CatalogException, 
            NoSuchMethodException {
        
        Course course = new CatalogImpl().newCourse("PB162",
                "Programování v jazyce Java");

        assertNotNull("Metoda newCourse vraci null",course);

        assertEquals("Metoda newCourse vraci kurz s jinzm nez zadanym kodem",
                "PB162",course.getId());

        assertTrue("Metoda equals vraci false, i kdyz je jejim parametrem " +
                "instance kurzu se stejnym id (chyba v metode equals)",
                course.equals(new TestCourse(course.getId())));

        assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
                "instance kurzu s odlisnym id (chyba v metode equals)",
                course.equals(new TestCourse("PA165")));

        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
                    "hodnota null (chyba v metode CourseImpl.equals)",
                    course.equals(null));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CourseImpl.equals() s parametrem null," 
                    + " metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }

        try {
            assertFalse("Metoda equals vraci true, i kdyz je jejim parametrem " +
                    "objekt jineho typu (chyba v metode CourseImpl.equals)",
                    course.equals(new Object()));
        } catch (Exception ex) {
            fail("Kdyz se zavola metoda CourseImpl.equals() s parametrem " +
                    "jineho typu, metoda nevrati false, ale vyhodi vyjimku " + 
                    ex.getClass().getName());
        }

        assertFalse("Metoda CourseImpl.hashCode() neni prekryta, takze " +
                "porusuje obecny kontrakt metody Object.hashCode()",
                CourseImpl.class.getMethod("hashCode").equals(
                Object.class.getMethod("hashCode")));

        Catalog catalog = new CatalogImpl();
        Course course2 = catalog.newCourse("PV168","Seminář z programování v " +
                "jazyce Java");                        
        Course course3 = new CatalogImpl().newCourse("PB162",
                "Jiný název");

        assertEquals("Metoda CourseImpl.hashCode() vraci pro dva stejne kurzy" +
                " ruznou hodnotu, takze porusuje obecny kontrakt metody " +
                "Object.hashCode() ",
                course.hashCode(),course3.hashCode());

        assertFalse("Metoda CourseImpl.hashCode() vraci pro dva ruzne kurzy " +
                "stejny kod; to sice neporusuje obecny kontrakt metody " +
                "Object.hashCode(), ale v tomto pripade lze snadno vytvorit " +
                "temer bezkolizni implementaci teto metody",
                course.hashCode() == course2.hashCode());
    }

    public void testNewCourse() throws CatalogException {
        Catalog catalog = new CatalogImpl();
        Course course = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");

        assertNotNull("Metoda newCourse vraci null",course);

        assertEquals("Nove vytvoreny kurz ma jine jmeno, nez bylo zadano " +
                "jako parametr metody CatalogImpl.newCourse()",
                "Moderni znackovaci jazyky a jejich aplikace",
                course.getName());

        assertEquals("Nove vytvoreny student ma jiny kod, nez byl zadan " +
                "jako parametr metody CatalogImpl.newCourse()",
                "PB138",
                course.getId());

        assertTrue("Mnozina kurzu vracena metodou getCourses() neobsahuje " +
                "nove vytvoreny kurz",catalog.getCourses().contains(course));

        try {
            catalog.newCourse("PB138","Jiny kurz");
            fail("Pokus o pridani kurzu s jiz existujicim kodem nevyvola " +
                    "zadnou vyjimku");
        } 
        catch (CatalogException ex) {
            // All is ok
            assertException("pridani kurzu s existujicim kodem",ex);
        }
        catch (Exception ex) {
            fail("Pokus o pridani kurzu s jiz existujicim kodem (id) vyvola " +
                    "misto CatalogException vyjimku " + 
                    ex.getClass().getName());
        }

        try {
            catalog.newCourse("PB 138","Jiny kurz");
            fail("Pokus o pridani kurzu, jehoz kod obsahuje mezeru, nevyvola " +
                    "zadnou vyjimku");
        } catch (IllegalArgumentException ex) {
            // All is ok
            assertException("pridani kurzu s kodem obsahujicim mezeru",ex);
        } catch (Exception ex) {
            fail("Pokus o pridani kurzu, jehoz kod (id) obsahuje mezeru," +
                    " vyvola misto IllegalArgumentException vyjimku " +
                    ex.getClass().getName());
        }
        
        try {
            catalog.newCourse(null,"Jiny kurz");
            fail("Pokus o pridani kurzu, jehoz kod je null, nevyvola " +
                    "zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("pridani kurzu s kodem null",ex);
        } catch (Exception ex) {
            fail("Pokus o pridani kurzu, jehoz kod (id) je null," +
                    " vyvola misto NullPointerException vyjimku " +
                    ex.getClass().getName());
        }

        try {
            catalog.newCourse("XX123",null);
            fail("Pokus o pridani kurzu, jehoz nazev je null, nevyvola " +
                    "zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("pridani kurzu s nazvem null",ex);
        } catch (Exception ex) {
            fail("Pokus o pridani kurzu, jehoz nazev je null," +
                    " vyvola misto NullPointerException vyjimku " +
                    ex.getClass().getName());
        }
    }
    
    public void testNewCatalog() throws CatalogException {
        Catalog catalog1 = new CatalogImpl();
        Catalog catalog2 = new CatalogImpl();
	assertNotNull("Metoda CatalogImpl.getStudents() vraci null, ackoliv null nikdy vracet nema",
	    catalog1.getStudents());
	assertNotNull("Metoda CatalogImpl.getCourses() vraci null, ackoliv null nikdy vracet nema",
	    catalog1.getCourses());
	assertTrue("Metoda CatalogImpl.getStudents() vraci neprazdnou kolekci, " +
	    "ackoliv jeste nebyl vytvoren zadny student",catalog1.getStudents().isEmpty());
	assertTrue("Metoda CatalogImpl.getCourses() vraci neprazdnou kolekci, " +
	    "ackoliv jeste nebyl vytvoren zadny predmet",catalog1.getCourses().isEmpty());
	catalog2.newStudent("Test student");
	catalog2.newCourse("A001","Test course");
	assertTrue("Metoda CatalogImpl.getStudents() vraci neprazdnou kolekci, " +
	    "ackoliv jeste nebyl vytvoren zadny student (pravdepodobne mate kolekci " +
	    "studentu realizovanou jako staticky atribut)",catalog1.getStudents().isEmpty());	
	assertTrue("Metoda CatalogImpl.getCourses() vraci neprazdnou kolekci, " +
	    "ackoliv jeste nebyl vytvoren zadny predmet (pravdepodobne mate kolekci " +
	    "predmetu realizovanou jako staticky atribut)",catalog1.getCourses().isEmpty());	

    }
    
    public void testGetCoursesOfNonExistentStudent() {
        try {
            new CatalogImpl().getCourses(null);
            fail("Volani metody getCourses s parametrem null " +
                    "nezpusobi zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("volani metody getCourses s parametrem null",ex);
        } catch (Exception ex) {
            fail("Volani metody getCourses s parametrem null vyvola " +
                    "misto NullPointerException vyjimku " + 
                    ex.getClass().getName());
        }
        
        try {
            new CatalogImpl().getCourses(new TestStudent(234));
            fail("Volani metody getCourses pro neexistujiciho studenta " +
                    "nezpusobi zadnou vyjimku");
        } catch (CatalogException ex) {
            // All is ok
            assertException("volani metody getCourses pro neexistujiciho studenta",ex);
        } catch (Exception ex) {
            fail("Volani metody getCourses pro neexistujiciho studenta vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }
    }

    public void testGetStudentsOfNonExistentCourse() {
        try {
            new CatalogImpl().getStudents(null);
            fail("Volani metody getStudents s parametrem null " +
                    "nezpusobi zadnou vyjimku");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("volani metody getStudents s parametrem null",ex);
        } catch (Exception ex) {
            fail("Volani metody getStudents s parametrem null vyvola " +
                    "misto NullPointerException vyjimku " + ex.getClass().getName());
        }

        try {
            new CatalogImpl().getStudents(new TestCourse("XXX"));
            fail("Volani metody getStudents pro neexistujici kurz " +
                    "nezpusobi zadnou vyjimku");
        } catch (CatalogException ex) {
            // All is ok
            assertException("volani metody getStudents pro neexistujici kurz",ex);
        } catch (Exception ex) {
            fail("Volani metody getStudents pro neexistujici kurz vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }
    }

    public void testRegister() throws CatalogException {

        Catalog catalog = new CatalogImpl();
        Course course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        Student student1 = catalog.newStudent("Testovaci student");
        Student student2 = catalog.newStudent("Testovaci student 2");

        Course course2 = catalog.newCourse("PA165","Tvorba programovych " +
                "systemu v jazyce Java");
        Course course3 = catalog.newCourse("PB162","Programovani v jazyce Java");

        Course strangeCourse = new TestCourse("Strange Course");
        Student strangeStudent = new TestStudent(student2.getId() + 1222);

        // tyto promenne obsahuji kurzy, ktere by mely mit
        // dani studenti zapsany. Po kazde operaci jejich 
        // obsah zaktualizujeme a zkontrolujeme, jestli
        // odpovida skutecnosti
        Set<Course> student1courses = new HashSet<Course>();
        Set<Course> student2courses = new HashSet<Course>();
        Set<Student> course1students = new HashSet<Student>();
        Set<Student> course2students = new HashSet<Student>();
        Set<Student> course3students = new HashSet<Student>();

        assertEquals("Seznam zapsanych kurzu studenta "+ student1 +" neodpovida",
                student1courses,catalog.getCourses(student1));
        assertEquals("Seznam zapsanych kurzu studenta "+ student2 +" neodpovida",
                student2courses,catalog.getCourses(student2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course1 +" neodpovida",
                course1students,catalog.getStudents(course1));
        assertEquals("Seznam zapsanych studentu kurzu "+ course2 +" neodpovida",
                course2students,catalog.getStudents(course2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course3 +" neodpovida",
                course3students,catalog.getStudents(course3));

        catalog.register(student1,course1,course2);
        student1courses.add(course1);
        student1courses.add(course2);
        course1students.add(student1);
        course2students.add(student1);

        assertEquals("Seznam zapsanych kurzu studenta "+ student1 +" neodpovida",
                student1courses,catalog.getCourses(student1));
        assertEquals("Seznam zapsanych kurzu studenta "+ student2 +" neodpovida",
                student2courses,catalog.getCourses(student2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course1 +" neodpovida",
                course1students,catalog.getStudents(course1));
        assertEquals("Seznam zapsanych studentu kurzu "+ course2 +" neodpovida",
                course2students,catalog.getStudents(course2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course3 +" neodpovida",
                course3students,catalog.getStudents(course3));

        catalog.register(student1,course3);
        student1courses.add(course3);
        course3students.add(student1);

        assertEquals("Seznam zapsanych kurzu studenta "+ student1 +" neodpovida",
                student1courses,catalog.getCourses(student1));
        assertEquals("Seznam zapsanych kurzu studenta "+ student2 +" neodpovida",
                student2courses,catalog.getCourses(student2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course1 +" neodpovida",
                course1students,catalog.getStudents(course1));
        assertEquals("Seznam zapsanych studentu kurzu "+ course2 +" neodpovida",
                course2students,catalog.getStudents(course2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course3 +" neodpovida",
                course3students,catalog.getStudents(course3));

        catalog.register(student2,course3);
        student2courses.add(course3);
        course3students.add(student2);

        assertEquals("Seznam zapsanych kurzu studenta "+ student1 +" neodpovida",
                student1courses,catalog.getCourses(student1));
        assertEquals("Seznam zapsanych kurzu studenta "+ student2 +" neodpovida",
                student2courses,catalog.getCourses(student2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course1 +" neodpovida",
                course1students,catalog.getStudents(course1));
        assertEquals("Seznam zapsanych studentu kurzu "+ course2 +" neodpovida",
                course2students,catalog.getStudents(course2));
        assertEquals("Seznam zapsanych studentu kurzu "+ course3 +" neodpovida",
                course3students,catalog.getStudents(course3));

        try {
            catalog.register(student1,course2);
            fail("Je mozne podruhe studenta zapsat do stejneho kurzu");
        } 
        catch (CatalogException ex) {
            // All is ok
            assertException("opakovany zapis studenta do stejneho kurzu",ex);
        }
        catch (Exception ex) {
            fail("Pokus o zapsani studenta podruhe do stejneho kurzu vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }

        try {
            catalog.register(student1,strangeCourse);
            fail("Je mozne studenta zapsat do ciziho (neexistujiciho) kurzu");
        } 
        catch (CatalogException ex) {
            // All is ok
            assertException("zapis studenta do neexistujiciho kurzu",ex);
        }
        catch (Exception ex) {
            fail("Pokus o zapsani studenta do ciziho (neexistujiciho) kurzu vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }

        try {
            catalog.register(strangeStudent,course1);
            fail("Je mozne do kurzu zapsat ciziho (neexistujiciho) studenta");
        } catch (CatalogException ex) {
            // All is ok
            assertException("zapis ciziho (neexistujiciho studenta) do " +
                    "nejakeho kurzu",ex);
        }
        catch (Exception ex) {
            fail("Pokus o zapsani ciziho (neexistujiciho) studenta vyvola " +
                    "misto CatalogException vyjimku " + ex.getClass().getName());
        }

        try {
            catalog.register(student1,(Course)null);
            fail("Je mozne zapsat studenta do kurzu null");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("zapis studenta do kurzu null",ex);
        }
        catch (Exception ex) {
            fail("Pokus o zapsani studenta do kurzu null vyvola " +
                    "misto NullPointerException vyjimku " + ex.getClass().getName());
        }

        try {
            catalog.register(null,course1);
            fail("Je mozne do kurzu zapsat studenta null");
        } catch (NullPointerException ex) {
            // All is ok
            assertException("zapis studenta null do nejakeho kurzu",ex);
        }
        catch (Exception ex) {
            fail("Pokus o zapsani studenta null vyvola " +
                    "misto NullPointerException vyjimku " + ex.getClass().getName());
        }

    }

    public void testAtomicFailure() throws CatalogException {

        // Atomic failure pro course = null
        Catalog catalog = new CatalogImpl();
        Course course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        Student student1 = catalog.newStudent("Testovaci student");
        assertTrue(catalog.getCourses(student1).isEmpty());
        assertTrue(catalog.getStudents(course1).isEmpty());
        try {
            catalog.register(student1,course1,null);
	    fail("Metoda register nevyhodi vyjimku NullPointerException pri" +
	        " pokusu o predani parametru course s hodnotou null");
        } catch (NullPointerException ex) {
            // All is ok
        }
        assertTrue("Pokud se pokusim zaregistrovat vice kurzu, z nichz jeden " +
                "je spatny (konkretne null), nektere kurzy se presto " +
                "zaregistruji (metoda neselze atomicky)",
                catalog.getCourses(student1).isEmpty() &&
                catalog.getStudents(course1).isEmpty());

        // Atomic failure pro neexistujici kurz 
        catalog = new CatalogImpl();
        course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        student1 = catalog.newStudent("Testovaci student");
        Course strangeCourse = new TestCourse("Strange Course");
        assertTrue(catalog.getCourses(student1).isEmpty());
        assertTrue(catalog.getStudents(course1).isEmpty());
        try {
            catalog.register(student1,course1,strangeCourse);
	    fail("Metoda register nevyhodi vyjimku CatalogException pri" +
	        " pokusu o zaregistrovani neexistujiciho kurzu");
        } catch (CatalogException ex) {
            // All is ok
        }
        assertTrue("Pokud se pokusim zaregistrovat vice kurzu, z nichz jeden " +
                "je spatny (konkretne neexistujici), nektere kurzy se presto " +
                "zaregistruji (metoda neselze atomicky)",
                catalog.getCourses(student1).isEmpty() &&
                catalog.getStudents(course1).isEmpty());

        // Atomic failure pro opakovany zapis kurzu 
        catalog = new CatalogImpl();
        course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        student1 = catalog.newStudent("Testovaci student");
        Course course2 = catalog.newCourse("PA165","Tvorba programovych " +
                "systemu v jazyce Java");
        assertTrue(catalog.getCourses(student1).isEmpty());
        assertTrue(catalog.getStudents(course1).isEmpty());
        catalog.register(student1,course1);
        try {
            catalog.register(student1,course2,course1);
        } catch (CatalogException ex) {
            // All is ok
        }
        assertTrue("Pokud se pokusim zaregistrovat vice kurzu, z nichz jeden " +
                "je jiz zaregistrovany, nektere kurzy se presto " +
                "zaregistruji (metoda neselze atomicky)",
                catalog.getCourses(student1).size() == 1 &&
                catalog.getCourses(student1).contains(course1) &&
                catalog.getStudents(course1).size() == 1 && 
                catalog.getStudents(course1).contains(student1));

        // Atomic failure pro vicenasobny zapis kurzu 
        catalog = new CatalogImpl();
        course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        student1 = catalog.newStudent("Testovaci student");
        course2 = catalog.newCourse("PA165","Tvorba programovych " +
                "systemu v jazyce Java");
        assertTrue(catalog.getCourses(student1).isEmpty());
        assertTrue(catalog.getStudents(course1).isEmpty());
        try {
            catalog.register(student1,course2,course1,course1);
            fail("Je mozne pro jednoho studenta zaregistrovat stejny kurz " +
                    "vicekrat (catalog.register(student1,course2,course1,course1))");
        } catch (CatalogException ex) {
            // All is ok
            assertException("vicenasobnou registraci stejneho kurzu",ex);
        }
        assertTrue("Pokud se pokusim zaregistrovat vice kurzu, z nichz nektery" +
                " je uveden vicekrat, nektere kurzy se presto " +
                "zaregistruji (metoda neselze atomicky)",
                catalog.getCourses(student1).isEmpty() &&
                catalog.getStudents(course1).isEmpty());
    }


    public final void testClear() throws CatalogException {
        Catalog catalog = new CatalogImpl();
        Student student = catalog.newStudent("test");
        assertFalse("Po pridani studenta je katalog stale prazdny",
                catalog.getStudents().isEmpty());
        Course course = catalog.newCourse("PB138","Moderni znackovaci jazyky " +
                "a jejich aplikace");
        assertFalse("Po pridani predmetu je katalog stale prazdny",
                catalog.getCourses().isEmpty());
        catalog.register(student,course);

        catalog.clear();
        assertTrue("Po volani metody clear neni katalog studentu prazdny",
                catalog.getStudents().isEmpty());
        catalog.clear();
        assertTrue("Po volani metody clear neni katalog predmetu prazdny",
                catalog.getCourses().isEmpty());
        try {
            catalog.getStudents(course);
            fail("Volani metody clear nevymaze registrovane studenty u predmetu");
        } catch(CatalogException ex) {
            // All is ok
        }

        try {
            catalog.getCourses(student);
            fail("Volani metody clear nevymaze registrovane predmety u studenta");
        } catch(CatalogException ex) {
            // All is ok
        }
    }    

    public void testReadOnlyCollections() throws CatalogException {
        
        Catalog catalog = new CatalogImpl();
        try {
            catalog.getStudents().clear();
            fail("Metoda getStudents() nevraci read-only kolekci!");
        }
        catch(UnsupportedOperationException ex) {
            // Vsechno je v poradku
            // Toto je jediny pripad, kdy muze byt blok catch prazdny!!!
            // kdekoliv jinde je to hruba chyba!
        }
        try {
            catalog.getCourses().clear();
            fail("Metoda getCourses() nevraci read-only kolekci!");
        }
        catch(UnsupportedOperationException ex) {
            // Vsechno je v poradku
        }

        Course course1 = catalog.newCourse("PB138","Moderni znackovaci " +
                "jazyky a jejich aplikace");
        Student student1 = catalog.newStudent("Testovaci student");


        try {
            catalog.getCourses(student1).clear();
            fail("Metoda getCourses(Student) nevraci read-only kolekci!");
        }
        catch(UnsupportedOperationException ex) {
            // All is ok...
        }

        try {
            catalog.getStudents(course1).clear();
            fail("Metoda getStudents(Catalog) nevraci read-only kolekci!");
        }
        catch(UnsupportedOperationException ex) {
            // All is ok...
        }
    }

    public static void main(String[] args) {
        junit.textui.TestRunner.run(CatalogTest.class);
    }

    protected static class TestStudent implements Student {

        private int id;
        
        protected TestStudent(int id) {
            this.id = id;
        }
        
        public String getName() {
            return "Student cislo " + id;
        }

        public int getId() {
            return id;
        }

        public int compareTo(Student o) {
            throw new UnsupportedOperationException();
        }
        
    }
    
    private class TestCourse implements Course {

        String code;

        TestCourse(String code) {
            this.code = code;
        }

        public String getName() {
            return "Test course";
        }

        public String getId() {
            return code;
        }

    }

}
