package cz.muni.fi.pb162.school;

/**
 * Rozhrani Course reprezentuje kurz. Kurz je 
 * identifikovan unikatnim indentifikatorem (kodem) id typu String. 
 * Dalsim atributem je jeho jmeno.
 *
 * @author Petr Adamek &lt;xadamek2@fi.muni.cz&gt;
 */
public interface Course {

    /**
     * Tato metoda vraci jmeno kurzu.
     *
     * @return jmeno kurzu
     */
    String getName();
    
    /**
     * Tato metoda vraci identifikator (kod) kurzu.
     *
     * @return identifikator kurzu
     */
    String getId();
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako tento kurz.
     * Jiny objekt je stejny (ekvivalentni) jako tento kurz prave
     * tehdy, pokud jde take o kurz a ten ma stejny identifikator 
     * id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();

}
