package cz.muni.fi.pb162.school;

/**
 * Rozhrani Student reprezentuje studenta. Student je 
 * identifikovan unikatnim indentifikatorem id typu int. 
 * Dalsim atributem je jeho jmeno. 
 * 
 * Toto rozhrani implementuje rozhrani Comparable,
 * pricemz poradi studentu je dano jejich identifikatory.
 *
 * @author Petr Adamek &lt;xadamek2@fi.muni.cz&gt;
 */
public interface Student extends Comparable<Student> {
    
    /**
     * Tato metoda vraci jmeno studenta.
     *
     * @return jmeno studenta
     */
    String getName();
    
    /**
     * Tato metoda vraci identifikator studenta.
     *
     * @return identifikator studenta
     */
    int getId();
    
    /**
     * <p>Indikuje, zdali je nejaky objekt stejny jako tento student.
     * Jiny objekt je stejny (ekvivalentni) jako tento student prave
     * tehdy, pokud jde take o studenta a ten ma stejny identifikator 
     * id.</p>
     * 
     * <p>Tato metoda musi samozrejme splnovat obecny kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#equals(java.lang.Object)">Object.equals</a>.</p>
     *
     * @param obj Porovnavany objekt
     * @return true, jestli je tento objekt stejny jako argument obj
     */
    boolean equals(Object obj);
    
    
    /**
     * Tato metoda vraci hashovaci kod daneho objektu. Metoda musi splnovat obecny 
     * kontrakt metody <a 
     * href="http://java.sun.com/j2se/1.5.0/docs/api/java/lang/Object.html#hashCode()">Object.hashCode()</a>.
     * 
     * @return hashovaci k daneho objektu
     */
    int hashCode();
    
}
